@@ -321,6 +321,7 @@ class DateTime(Column):
     db_type = 'timestamp'
 
     def to_python(self, value):
+        if value is None: return
         if isinstance(value, datetime):
             return value
         elif isinstance(value, date):
@@ -346,6 +347,7 @@ class Date(Column):
 
 
     def to_python(self, value):
+        if value is None: return
         if isinstance(value, datetime):
             return value.date()
         elif isinstance(value, date):
@@ -355,6 +357,7 @@ class Date(Column):
 
     def to_database(self, value):
         value = super(Date, self).to_database(value)
+        if value is None: return
         if isinstance(value, datetime):
             value = value.date()
         if not isinstance(value, date):
@@ -644,7 +647,7 @@ class List(BaseContainerColumn):
         def __nonzero__(self):
             return bool(self.value)
 
-    def __init__(self, value_type, default=set, **kwargs):
+    def __init__(self, value_type, default=list, **kwargs):
         return super(List, self).__init__(value_type=value_type, default=default, **kwargs)
 
     def validate(self, value):
@@ -865,8 +868,15 @@ class _PartitionKeysToken(Column):
         self.partition_columns = model._partition_keys.values()
         super(_PartitionKeysToken, self).__init__(partition_key=True)
 
+    @property
+    def db_field_name(self):
+        return 'token({})'.format(', '.join(['"{}"'.format(c.db_field_name) for c in self.partition_columns]))
+
     def to_database(self, value):
-        raise NotImplementedError
+        from cqlengine.functions import Token
+        assert isinstance(value, Token)
+        value.set_columns(self.partition_columns)
+        return value
 
     def get_cql(self):
         return "token({})".format(", ".join(c.cql for c in self.partition_columns))
