@@ -69,6 +69,7 @@ class QuerySetDescriptor(object):
         """
         raise NotImplementedError
 
+
 class TTLDescriptor(object):
     """
     returns a query set descriptor
@@ -128,7 +129,11 @@ class ColumnQueryEvaluator(AbstractQueryableColumn):
     def __init__(self, column):
         self.column = column
 
+    def __unicode__(self):
+        return self.column.db_field_name
+
     def _get_column(self):
+        """ :rtype: ColumnQueryEvaluator """
         return self.column
 
 
@@ -247,6 +252,17 @@ class BaseModel(object):
         self._is_persisted = False
         self._batch = None
 
+
+    def __repr__(self):
+        """
+        Pretty printing of models by their primary key
+        """
+        return '{} <{}>'.format(self.__class__.__name__, 
+                                ', '.join(('{}={}'.format(k, getattr(self, k)) for k,v in self._primary_keys.iteritems()))
+                                )
+
+    
+
     @classmethod
     def _discover_polymorphic_submodels(cls):
         if not cls._is_polymorphic_base:
