@@ -1,169 +1,25 @@
 import copy
 from datetime import datetime
-from uuid import uuid4
-from hashlib import md5
-from time import time
-from uuid import uuid1
-from cqlengine import BaseContainerColumn, BaseValueManager, Map, columns
-from cqlengine.columns import Counter
+from cqlengine import BaseContainerColumn, Map, columns
+from cqlengine.columns import Counter, List, Set
 
-from cqlengine.connection import connection_manager, execute, RowResult
+from cqlengine.connection import execute, RowResult
 
 from cqlengine.exceptions import CQLEngineException, ValidationError
-from cqlengine.functions import QueryValue, Token
+from cqlengine.functions import Token, BaseQueryFunction, QueryValue
 
 #CQL 3 reference:
 #http://www.datastax.com/docs/1.1/references/cql/index
+from cqlengine.operators import InOperator, EqualsOperator, GreaterThanOperator, GreaterThanOrEqualOperator
+from cqlengine.operators import LessThanOperator, LessThanOrEqualOperator, BaseWhereOperator
+from cqlengine.statements import WhereClause, SelectStatement, DeleteStatement, UpdateStatement, AssignmentClause, InsertStatement, BaseCQLStatement, MapUpdateClause, MapDeleteClause, ListUpdateClause, SetUpdateClause, CounterUpdateClause
+
 
 class QueryException(CQLEngineException): pass
 class DoesNotExist(QueryException): pass
 class MultipleObjectsReturned(QueryException): pass
 
 
-class QueryOperatorException(QueryException): pass
-
-
-class QueryOperator(object):
-    # The symbol that identifies this operator in filter kwargs
-    # ie: colname__<symbol>
-    symbol = None
-
-    # The comparator symbol this operator uses in cql
-    cql_symbol = None
-
-    QUERY_VALUE_WRAPPER = QueryValue
-
-    def __init__(self, column, value):
-        self.column = column
-        self.value = value
-
-        if isinstance(value, QueryValue):
-            self.query_value = value
-        else:
-            self.query_value = self.QUERY_VALUE_WRAPPER(value)
-
-        #perform validation on this operator
-        self.validate_operator()
-        self.validate_value()
-
-    @property
-    def cql(self):
-        """
-        Returns this operator's portion of the WHERE clause
-        """
-        return '{} {} {}'.format(self.column.cql, self.cql_symbol, self.query_value.cql)
-
-    def validate_operator(self):
-        """
-        Checks that this operator can be used on the column provided
-        """
-        if self.symbol is None:
-            raise QueryOperatorException(
-                    "{} is not a valid operator, use one with 'symbol' defined".format(
-                        self.__class__.__name__
-                    )
-                )
-        if self.cql_symbol is None:
-            raise QueryOperatorException(
-                    "{} is not a valid operator, use one with 'cql_symbol' defined".format(
-                        self.__class__.__name__
-                    )
-                )
-
-    def validate_value(self):
-        """
-        Checks that the compare value works with this operator
-
-        Doesn't do anything by default
-        """
-        pass
-
-    def get_dict(self):
-        """
-        Returns this operators contribution to the cql.query arg dictionanry
-
-        ie: if this column's name is colname, and the identifier is colval,
-        this should return the dict: {'colval':<self.value>}
-        SELECT * FROM column_family WHERE colname=:colval
-        """
-        return self.query_value.get_dict(self.column)
-
-    @classmethod
-    def get_operator(cls, symbol):
-        if not hasattr(cls, 'opmap'):
-            QueryOperator.opmap = {}
-            def _recurse(klass):
-                if klass.symbol:
-                    QueryOperator.opmap[klass.symbol.upper()] = klass
-                for subklass in klass.__subclasses__():
-                    _recurse(subklass)
-                pass
-            _recurse(QueryOperator)
-        try:
-            return QueryOperator.opmap[symbol.upper()]
-        except KeyError:
-            raise QueryOperatorException("{} doesn't map to a QueryOperator".format(symbol))
-
-    # equality operator, used by tests
-
-    def __eq__(self, op):
-        return self.__class__ is op.__class__ and \
-                self.column.db_field_name == op.column.db_field_name and \
-                self.value == op.value
-
-    def __ne__(self, op):
-        return not (self == op)
-
-    def __hash__(self):
-        return hash(self.column.db_field_name) ^ hash(self.value)
-
-
-class EqualsOperator(QueryOperator):
-    symbol = 'EQ'
-    cql_symbol = '='
-
-
-class IterableQueryValue(QueryValue):
-    def __init__(self, value):
-        try:
-            super(IterableQueryValue, self).__init__(value, [uuid4().hex for i in value])
-        except TypeError:
-            raise QueryException("in operator arguments must be iterable, {} found".format(value))
-
-    def get_dict(self, column):
-        return dict((i, column.to_database(v)) for (i, v) in zip(self.identifier, self.value))
-
-    def get_cql(self):
-        return '({})'.format(', '.join(':{}'.format(i) for i in self.identifier))
-
-
-class InOperator(EqualsOperator):
-    symbol = 'IN'
-    cql_symbol = 'IN'
-
-    QUERY_VALUE_WRAPPER = IterableQueryValue
-
-
-class GreaterThanOperator(QueryOperator):
-    symbol = "GT"
-    cql_symbol = '>'
-
-
-class GreaterThanOrEqualOperator(QueryOperator):
-    symbol = "GTE"
-    cql_symbol = '>='
-
-
-class LessThanOperator(QueryOperator):
-    symbol = "LT"
-    cql_symbol = '<'
-
-
-class LessThanOrEqualOperator(QueryOperator):
-    symbol = "LTE"
-    cql_symbol = '<='
-
-
 class AbstractQueryableColumn(object):
     """
     exposes cql query operators through pythons
@@ -173,28 +29,40 @@ class AbstractQueryableColumn(object):
     def _get_column(self):
         raise NotImplementedError
 
+    def __unicode__(self):
+        raise NotImplementedError
+
+    def __str__(self):
+        return str(unicode(self))
+
+    def _to_database(self, val):
+        if isinstance(val, QueryValue):
+            return val
+        else:
+            return self._get_column().to_database(val)
+
     def in_(self, item):
         """
         Returns an in operator
 
-        used in where you'd typically want to use python's `in` operator
+        used where you'd typically want to use python's `in` operator
         """
-        return InOperator(self._get_column(), item)
+        return WhereClause(unicode(self), InOperator(), item)
 
     def __eq__(self, other):
-        return EqualsOperator(self._get_column(), other)
+        return WhereClause(unicode(self), EqualsOperator(), self._to_database(other))
 
     def __gt__(self, other):
-        return GreaterThanOperator(self._get_column(), other)
+        return WhereClause(unicode(self), GreaterThanOperator(), self._to_database(other))
 
     def __ge__(self, other):
-        return GreaterThanOrEqualOperator(self._get_column(), other)
+        return WhereClause(unicode(self), GreaterThanOrEqualOperator(), self._to_database(other))
 
     def __lt__(self, other):
-        return LessThanOperator(self._get_column(), other)
+        return WhereClause(unicode(self), LessThanOperator(), self._to_database(other))
 
     def __le__(self, other):
-        return LessThanOrEqualOperator(self._get_column(), other)
+        return WhereClause(unicode(self), LessThanOrEqualOperator(), self._to_database(other))
 
 
 class BatchType(object):
@@ -218,8 +86,10 @@ class BatchQuery(object):
         self.timestamp = timestamp
         self._consistency = consistency
 
-    def add_query(self, query, params):
-        self.queries.append((query, params))
+    def add_query(self, query):
+        if not isinstance(query, BaseCQLStatement):
+            raise CQLEngineException('only BaseCQLStatements can be added to a batch query')
+        self.queries.append(query)
 
     def consistency(self, consistency):
         self._consistency = consistency
@@ -237,9 +107,13 @@ class BatchQuery(object):
 
         query_list = [opener]
         parameters = {}
-        for query, params in self.queries:
-            query_list.append('  ' + query)
-            parameters.update(params)
+        ctx_counter = 0
+        for query in self.queries:
+            query.update_context_id(ctx_counter)
+            ctx = query.get_context()
+            ctx_counter += len(ctx)
+            query_list.append('  ' + str(query))
+            parameters.update(ctx)
 
         query_list.append('APPLY BATCH;')
 
@@ -295,8 +169,14 @@ class AbstractQuerySet(object):
     def column_family_name(self):
         return self.model.column_family_name()
 
+    def _execute(self, q):
+        if self._batch:
+            return self._batch.add_query(q)
+        else:
+            return execute(q, consistency_level=self._consistency)
+
     def __unicode__(self):
-        return self._select_query()
+        return unicode(self._select_query())
 
     def __str__(self):
         return str(self.__unicode__())
@@ -306,7 +186,7 @@ class AbstractQuerySet(object):
 
     def __deepcopy__(self, memo):
         clone = self.__class__(self.model)
-        for k,v in self.__dict__.items():
+        for k, v in self.__dict__.items():
             if k in ['_con', '_cur', '_result_cache', '_result_idx']: # don't clone these
                 clone.__dict__[k] = None
             elif k == '_batch':
@@ -326,41 +206,27 @@ class AbstractQuerySet(object):
 
     #----query generation / execution----
 
-    def _where_clause(self):
-        """ Returns a where clause based on the given filter args """
-        return ' AND '.join([f.cql for f in self._where])
+    def _select_fields(self):
+        """ returns the fields to select """
+        return []
 
-    def _where_values(self):
-        """ Returns the value dict to be passed to the cql query """
-        values = {}
-        for where in self._where:
-            values.update(where.get_dict())
-        return values
-
-    def _get_select_statement(self):
-        """ returns the select portion of this queryset's cql statement """
-        raise NotImplementedError
+    def _validate_select_where(self):
+        """ put select query validation here """
 
     def _select_query(self):
         """
         Returns a select clause based on the given filter args
         """
-        qs = [self._get_select_statement()]
-        qs += ['FROM {}'.format(self.column_family_name)]
-
         if self._where:
-            qs += ['WHERE {}'.format(self._where_clause())]
-
-        if self._order:
-            qs += ['ORDER BY {}'.format(', '.join(self._order))]
-
-        if self._limit:
-            qs += ['LIMIT {}'.format(self._limit)]
-
-        if self._allow_filtering:
-            qs += ['ALLOW FILTERING']
-
-        return ' '.join(qs)
+            self._validate_select_where()
+        return SelectStatement(
+            self.column_family_name,
+            fields=self._select_fields(),
+            where=self._where,
+            order_by=self._order,
+            limit=self._limit,
+            allow_filtering=self._allow_filtering
+        )
 
     #----Reads------
 
@@ -368,7 +234,7 @@ class AbstractQuerySet(object):
         if self._batch:
             raise CQLEngineException("Only inserts, updates, and deletes are available in batch mode")
         if self._result_cache is None:
-            columns, self._result_cache = execute(self._select_query(), self._where_values(), self._consistency)
+            columns, self._result_cache = self._execute(self._select_query())
             self._construct_result = self._get_result_constructor(columns)
 
     def _fill_result_cache_to_idx(self, idx):
@@ -477,26 +343,37 @@ class AbstractQuerySet(object):
         #add arguments to the where clause filters
         clone = copy.deepcopy(self)
         for operator in args:
-            if not isinstance(operator, QueryOperator):
+            if not isinstance(operator, WhereClause):
                 raise QueryException('{} is not a valid query operator'.format(operator))
             clone._where.append(operator)
 
         for arg, val in kwargs.items():
             col_name, col_op = self._parse_filter_arg(arg)
+            quote_field = True
             #resolve column and operator
             try:
                 column = self.model._get_column(col_name)
             except KeyError:
                 if col_name == 'pk__token':
                     column = columns._PartitionKeysToken(self.model)
+                    quote_field = False
                 else:
                     raise QueryException("Can't resolve column name: '{}'".format(col_name))
 
             #get query operator, or use equals if not supplied
-            operator_class = QueryOperator.get_operator(col_op or 'EQ')
-            operator = operator_class(column, val)
+            operator_class = BaseWhereOperator.get_operator(col_op or 'EQ')
+            operator = operator_class()
 
-            clone._where.append(operator)
+            if isinstance(operator, InOperator):
+                if not isinstance(val, (list, tuple)):
+                    raise QueryException('IN queries must use a list/tuple value')
+                query_val = [column.to_database(v) for v in val]
+            elif isinstance(val, BaseQueryFunction):
+                query_val = val
+            else:
+                query_val = column.to_database(val)
+
+            clone._where.append(WhereClause(column.db_field_name, operator, query_val, quote_field=quote_field))
 
         return clone
 
@@ -549,18 +426,11 @@ class AbstractQuerySet(object):
         """ Returns the number of rows matched by this query """
         if self._batch:
             raise CQLEngineException("Only inserts, updates, and deletes are available in batch mode")
-        #TODO: check for previous query execution and return row count if it exists
+
         if self._result_cache is None:
-            qs = ['SELECT COUNT(*)']
-            qs += ['FROM {}'.format(self.column_family_name)]
-            if self._where:
-                qs += ['WHERE {}'.format(self._where_clause())]
-            if self._allow_filtering:
-                qs += ['ALLOW FILTERING']
-
-            qs = ' '.join(qs)
-
-            _, result = execute(qs, self._where_values())
+            query = self._select_query()
+            query.count = True
+            _, result = self._execute(query)
             return result[0][0]
         else:
             return len(self._result_cache)
@@ -623,26 +493,25 @@ class AbstractQuerySet(object):
     def create(self, **kwargs):
         return self.model(**kwargs).batch(self._batch).ttl(self._ttl).consistency(self._consistency).save()
 
-    #----delete---
-    def delete(self, columns=[]):
+    def delete(self):
         """
         Deletes the contents of a query
         """
         #validate where clause
         partition_key = self.model._primary_keys.values()[0]
-        if not any([c.column.db_field_name == partition_key.db_field_name for c in self._where]):
+        if not any([c.field == partition_key.column_name for c in self._where]):
             raise QueryException("The partition key must be defined on delete queries")
-        qs = ['DELETE FROM {}'.format(self.column_family_name)]
-        qs += ['WHERE {}'.format(self._where_clause())]
-        qs = ' '.join(qs)
 
-        if self._batch:
-            self._batch.add_query(qs, self._where_values())
-        else:
-            execute(qs, self._where_values())
+        dq = DeleteStatement(
+            self.column_family_name,
+            where=self._where
+        )
+        self._execute(dq)
 
     def __eq__(self, q):
-        return set(self._where) == set(q._where)
+        if len(self._where) == len(q._where):
+            return all([w in q._where for w in self._where])
+        return False
 
     def __ne__(self, q):
         return not (self != q)
@@ -665,10 +534,6 @@ class SimpleQuerySet(AbstractQuerySet):
 
     """
 
-    def _get_select_statement(self):
-        """ Returns the fields to be returned by the select query """
-        return 'SELECT *'
-
     def _get_result_constructor(self, names):
         """
         Returns a function that will be used to instantiate query results
@@ -682,40 +547,31 @@ class ModelQuerySet(AbstractQuerySet):
     """
 
     """
-    def _validate_where_syntax(self):
-        """ Checks that a filterset will not create invalid cql """
-
+    def _validate_select_where(self):
+        """ Checks that a filterset will not create invalid select statement """
         #check that there's either a = or IN relationship with a primary key or indexed field
-        equal_ops = [w for w in self._where if isinstance(w, EqualsOperator)]
-        token_ops = [w for w in self._where if isinstance(w.value, Token)]
-        if not any([w.column.primary_key or w.column.index for w in equal_ops]) and not token_ops:
+        equal_ops = [self.model._columns.get(w.field) for w in self._where if isinstance(w.operator, EqualsOperator)]
+        token_ops = [self.model._columns.get(w.field) for w in self._where if isinstance(w.operator, Token)]
+        if not any([w.primary_key or w.index for w in equal_ops]) and not token_ops:
             raise QueryException('Where clauses require either a "=" or "IN" comparison with either a primary key or indexed field')
 
         if not self._allow_filtering:
             #if the query is not on an indexed field
-            if not any([w.column.index for w in equal_ops]):
-                if not any([w.column.partition_key for w in equal_ops]) and not token_ops:
+            if not any([w.index for w in equal_ops]):
+                if not any([w.partition_key for w in equal_ops]) and not token_ops:
                     raise QueryException('Filtering on a clustering key without a partition key is not allowed unless allow_filtering() is called on the querset')
-            if any(not w.column.partition_key for w in token_ops):
+            if any(not w.partition_key for w in token_ops):
                 raise QueryException('The token() function is only supported on the partition key')
 
-    def _where_clause(self):
-        """ Returns a where clause based on the given filter args """
-        self._validate_where_syntax()
-        return super(ModelQuerySet, self)._where_clause()
-
-    def _get_select_statement(self):
-        """ Returns the fields to be returned by the select query """
+    def _select_fields(self):
         if self._defer_fields or self._only_fields:
             fields = self.model._columns.keys()
             if self._defer_fields:
                 fields = [f for f in fields if f not in self._defer_fields]
             elif self._only_fields:
                 fields = self._only_fields
-            db_fields = [self.model._columns[f].db_field_name for f in fields]
-            return 'SELECT {}'.format(', '.join(['"{}"'.format(f) for f in db_fields]))
-        else:
-            return 'SELECT *'
+            return [self.model._columns[f].db_field_name for f in fields]
+        return super(ModelQuerySet, self)._select_fields()
 
     def _get_result_constructor(self, names):
         """ Returns a function that will be used to instantiate query results """
@@ -747,11 +603,6 @@ class ModelQuerySet(AbstractQuerySet):
 
         return column.db_field_name, order_type
 
-    def _get_ttl_statement(self):
-        if not self._ttl:
-            return ""
-        return "USING TTL {}".format(self._ttl)
-
     def values_list(self, *fields, **kwargs):
         """ Instructs the query set to return tuples, not model instance """
         flat = kwargs.pop('flat', False)
@@ -780,9 +631,8 @@ class ModelQuerySet(AbstractQuerySet):
         if not values:
             return
 
-        set_statements = []
-        ctx = {}
         nulled_columns = set()
+        us = UpdateStatement(self.column_family_name, where=self._where, ttl=self._ttl)
         for name, val in values.items():
             col = self.model._columns.get(name)
             # check for nonexistant columns
@@ -797,33 +647,18 @@ class ModelQuerySet(AbstractQuerySet):
                 nulled_columns.add(name)
                 continue
             # add the update statements
-            if isinstance(col, (BaseContainerColumn, Counter)):
-                val_mgr = self.instance._values[name]
-                set_statements += col.get_update_statement(val, val_mgr.previous_value, ctx)
-
+            if isinstance(col, Counter):
+                # TODO: implement counter updates
+                raise NotImplementedError
             else:
-                field_id = uuid4().hex
-                set_statements += ['"{}" = :{}'.format(col.db_field_name, field_id)]
-                ctx[field_id] = val
+                us.add_assignment_clause(AssignmentClause(name, col.to_database(val)))
 
-        if set_statements:
-            ttl_stmt = "USING TTL {}".format(self._ttl) if self._ttl else ""
-            qs = "UPDATE {} SET {} WHERE {} {}".format(
-                self.column_family_name,
-                ', '.join(set_statements),
-                self._where_clause(),
-                ttl_stmt
-            )
-            ctx.update(self._where_values())
-            execute(qs, ctx, self._consistency)
+        if us.assignments:
+            self._execute(us)
 
         if nulled_columns:
-            qs = "DELETE {} FROM {} WHERE {}".format(
-                ', '.join(nulled_columns),
-                self.column_family_name,
-                self._where_clause()
-            )
-            execute(qs, self._where_values(), self._consistency)
+            ds = DeleteStatement(self.column_family_name, fields=nulled_columns, where=self._where)
+            self._execute(ds)
 
 
 class DMLQuery(object):
@@ -845,6 +680,12 @@ class DMLQuery(object):
         self._ttl = ttl
         self._consistency = consistency
 
+    def _execute(self, q):
+        if self._batch:
+            return self._batch.add_query(q)
+        else:
+            return execute(q, consistency_level=self._consistency)
+
     def batch(self, batch_obj):
         if batch_obj is not None and not isinstance(batch_obj, BatchQuery):
             raise CQLEngineException('batch_obj must be a BatchQuery instance or None')
@@ -855,39 +696,27 @@ class DMLQuery(object):
         """
         executes a delete query to remove columns that have changed to null
         """
-        values, field_names, field_ids, field_values, query_values = self._get_query_values()
-
-        # delete nulled columns and removed map keys
-        qs = ['DELETE']
-        query_values = {}
-
-        del_statements = []
-        for k,v in self.instance._values.items():
+        ds = DeleteStatement(self.column_family_name)
+        deleted_fields = False
+        for _, v in self.instance._values.items():
             col = v.column
             if v.deleted:
-                del_statements += ['"{}"'.format(col.db_field_name)]
+                ds.add_field(col.db_field_name)
+                deleted_fields = True
             elif isinstance(col, Map):
-                del_statements += col.get_delete_statement(v.value, v.previous_value, query_values)
+                uc = MapDeleteClause(col.db_field_name, v.value, v.previous_value)
+                if uc.get_context_size() > 0:
+                    ds.add_field(uc)
+                    deleted_fields = True
 
-        if del_statements:
-            qs += [', '.join(del_statements)]
-
-            qs += ['FROM {}'.format(self.column_family_name)]
-
-            qs += ['WHERE']
-            where_statements = []
+        if deleted_fields:
             for name, col in self.model._primary_keys.items():
-                field_id = uuid4().hex
-                query_values[field_id] = field_values[name]
-                where_statements += ['"{}" = :{}'.format(col.db_field_name, field_id)]
-            qs += [' AND '.join(where_statements)]
-
-            qs = ' '.join(qs)
-
-            if self._batch:
-                self._batch.add_query(qs, query_values)
-            else:
-                execute(qs, query_values)
+                ds.add_where_clause(WhereClause(
+                    col.db_field_name,
+                    EqualsOperator(),
+                    col.to_database(getattr(self.instance, name))
+                ))
+            self._execute(ds)
 
     def update(self):
         """
@@ -900,85 +729,50 @@ class DMLQuery(object):
             raise CQLEngineException("DML Query intance attribute is None")
         assert type(self.instance) == self.model
 
-        values, field_names, field_ids, field_values, query_values = self._get_query_values()
-
-        qs = []
-        qs += ["UPDATE {}".format(self.column_family_name)]
-        qs += ["SET"]
-
-        set_statements = []
+        statement = UpdateStatement(self.column_family_name, ttl=self._ttl)
         #get defined fields and their column names
         for name, col in self.model._columns.items():
             if not col.is_primary_key:
-                val = values.get(name)
+                val = getattr(self.instance, name, None)
+                val_mgr = self.instance._values[name]
 
                 # don't update something that is null
                 if val is None:
                     continue
 
                 # don't update something if it hasn't changed
-                if not self.instance._values[name].changed and not isinstance(col, Counter):
+                if not val_mgr.changed and not isinstance(col, Counter):
                     continue
 
-                # add the update statements
                 if isinstance(col, (BaseContainerColumn, Counter)):
-                    #remove value from query values, the column will handle it
-                    query_values.pop(field_ids.get(name), None)
-
-                    val_mgr = self.instance._values[name]
-                    set_statements += col.get_update_statement(val, val_mgr.previous_value, query_values)
+                    # get appropriate clause
+                    if isinstance(col, List): klass = ListUpdateClause
+                    elif isinstance(col, Map): klass = MapUpdateClause
+                    elif isinstance(col, Set): klass = SetUpdateClause
+                    elif isinstance(col, Counter): klass = CounterUpdateClause
+                    else: raise RuntimeError
 
+                    # do the stuff
+                    clause = klass(col.db_field_name, val, val_mgr.previous_value, column=col)
+                    if clause.get_context_size() > 0:
+                        statement.add_assignment_clause(clause)
                 else:
-                    set_statements += ['"{}" = :{}'.format(col.db_field_name, field_ids[col.db_field_name])]
-        qs += [', '.join(set_statements)]
+                    statement.add_assignment_clause(AssignmentClause(
+                        col.db_field_name,
+                        col.to_database(val)
+                    ))
 
-        qs += ['WHERE']
-
-        where_statements = []
-        for name, col in self.model._primary_keys.items():
-            where_statements += ['"{}" = :{}'.format(col.db_field_name, field_ids[col.db_field_name])]
-
-        qs += [' AND '.join(where_statements)]
-
-        if self._ttl:
-            qs += ["USING TTL {}".format(self._ttl)]
-
-        # clear the qs if there are no set statements and this is not a counter model
-        if not set_statements and not self.instance._has_counter:
-            qs = []
-
-        qs = ' '.join(qs)
-        # skip query execution if it's empty
-        # caused by pointless update queries
-        if qs:
-            if self._batch:
-                self._batch.add_query(qs, query_values)
-            else:
-                execute(qs, query_values, consistency_level=self._consistency)
+        if statement.get_context_size() > 0 or self.instance._has_counter:
+            for name, col in self.model._primary_keys.items():
+                statement.add_where_clause(WhereClause(
+                    col.db_field_name,
+                    EqualsOperator(),
+                    col.to_database(getattr(self.instance, name))
+                ))
+            self._execute(statement)
 
         self._delete_null_columns()
 
-    def _get_query_values(self):
-        """
-        returns all the data needed to do queries
-        """
-        #organize data
-        value_pairs = []
-        values = self.instance._as_dict()
-
-        #get defined fields and their column names
-        for name, col in self.model._columns.items():
-            val = values.get(name)
-            if col._val_is_null(val): continue
-            value_pairs += [(col.db_field_name, val)]
-
-        #construct query string
-        field_names = zip(*value_pairs)[0]
-        field_ids = {n:uuid4().hex for n in field_names}
-        field_values = dict(value_pairs)
-        query_values = {field_ids[n]:field_values[n] for n in field_names}
-        return values, field_names, field_ids, field_values, query_values
-
     def save(self):
         """
         Creates / updates a row.
@@ -990,31 +784,26 @@ class DMLQuery(object):
             raise CQLEngineException("DML Query intance attribute is None")
         assert type(self.instance) == self.model
 
-        values, field_names, field_ids, field_values, query_values = self._get_query_values()
-
-        qs = []
+        nulled_fields = set()
         if self.instance._has_counter or self.instance._can_update():
             return self.update()
         else:
-            qs += ["INSERT INTO {}".format(self.column_family_name)]
-            qs += ["({})".format(', '.join(['"{}"'.format(f) for f in field_names]))]
-            qs += ['VALUES']
-            qs += ["({})".format(', '.join([':'+field_ids[f] for f in field_names]))]
-
-        if self._ttl:
-            qs += ["USING TTL {}".format(self._ttl)]
-
-        qs += []
-        qs = ' '.join(qs)
-
+            insert = InsertStatement(self.column_family_name, ttl=self._ttl)
+            for name, col in self.instance._columns.items():
+                val = getattr(self.instance, name, None)
+                if col._val_is_null(val):
+                    if self.instance._values[name].changed:
+                        nulled_fields.add(col.db_field_name)
+                    continue
+                insert.add_assignment_clause(AssignmentClause(
+                    col.db_field_name,
+                    col.to_database(getattr(self.instance, name, None))
+                ))
 
         # skip query execution if it's empty
         # caused by pointless update queries
-        if qs:
-            if self._batch:
-                self._batch.add_query(qs, query_values)
-            else:
-                execute(qs, query_values, self._consistency)
+        if not insert.is_empty:
+            self._execute(insert)
 
         # delete any nulled columns
         self._delete_null_columns()
@@ -1022,22 +811,15 @@ class DMLQuery(object):
     def delete(self):
         """ Deletes one instance """
         if self.instance is None:
-            raise CQLEngineException("DML Query intance attribute is None")
-        field_values = {}
-        qs = ['DELETE FROM {}'.format(self.column_family_name)]
-        qs += ['WHERE']
-        where_statements = []
+            raise CQLEngineException("DML Query instance attribute is None")
+
+        ds = DeleteStatement(self.column_family_name)
         for name, col in self.model._primary_keys.items():
-            field_id = uuid4().hex
-            field_values[field_id] = col.to_database(getattr(self.instance, name))
-            where_statements += ['"{}" = :{}'.format(col.db_field_name, field_id)]
-
-        qs += [' AND '.join(where_statements)]
-        qs = ' '.join(qs)
-
-        if self._batch:
-            self._batch.add_query(qs, field_values)
-        else:
-            execute(qs, field_values, self._consistency)
+            ds.add_where_clause(WhereClause(
+                col.db_field_name,
+                EqualsOperator(),
+                col.to_database(getattr(self.instance, name))
+            ))
+        self._execute(ds)
 
 
